@@ -4,9 +4,10 @@ the method of parsing data from: https://github.com/saper-2/BRN-Printer-sCounter
 """
 import logging
 import re
+from datetime import datetime, timedelta
 
-from pysnmp.error import PySnmpError
 import pysnmp.hlapi.asyncio as hlapi
+from pysnmp.error import PySnmpError
 from pysnmp.hlapi.asyncore.cmdgen import lcd
 
 from .const import (
@@ -39,7 +40,7 @@ REGEX_MODEL_PATTERN = re.compile(r"MDL:(?P<model>[\w\-]+)")
 class Brother:  # pylint:disable=too-many-instance-attributes
     """Main class to perform snmp requests to printer."""
 
-    def __init__(self, host, port=161, kind="laser"):
+    def __init__(self, host, port=161, kind="laser", snmp_engine=None):
         """Initialize."""
         if kind not in KINDS:
             _LOGGER.warning("Wrong kind argument. 'laser' was used.")
@@ -56,13 +57,16 @@ class Brother:  # pylint:disable=too-many-instance-attributes
         self.serial = None
         self._host = host
         self._port = port
+        self._last_uptime = None
 
-        self._snmp_engine = None
+        self._snmp_engine = snmp_engine
         self._oids = tuple(self._iterate_oids(OIDS.values()))
 
         _LOGGER.debug("Using host: %s", host)
 
-    async def async_update(self):  # pylint:disable=too-many-branches
+    async def async_update(
+        self,
+    ):  # pylint:disable=too-many-branches,too-many-statements
         """Update data from printer."""
         raw_data = await self._get_data()
 
@@ -81,10 +85,10 @@ class Brother:  # pylint:disable=too-many-instance-attributes
             data[ATTR_MODEL] = self.model
             self.serial = raw_data[OIDS[ATTR_SERIAL]]
             data[ATTR_SERIAL] = self.serial
-        except (TypeError, AttributeError):
+        except (TypeError, AttributeError) as err:
             raise UnsupportedModel(
                 "It seems that this printer model is not supported. Sorry."
-            )
+            ) from err
         try:
             self.firmware = raw_data[OIDS[ATTR_FIRMWARE]]
             data[ATTR_FIRMWARE] = self.firmware
@@ -105,9 +109,22 @@ class Brother:  # pylint:disable=too-many-instance-attributes
         except (AttributeError, KeyError, TypeError):
             _LOGGER.debug("Incomplete data from printer.")
         try:
-            data[ATTR_UPTIME] = int(raw_data.get(OIDS[ATTR_UPTIME])) / 100
+            uptime = int(raw_data.get(OIDS[ATTR_UPTIME])) / 100
         except TypeError:
             pass
+        else:
+            if not self._last_uptime:
+                data[ATTR_UPTIME] = self._last_uptime = (
+                    datetime.utcnow() - timedelta(seconds=uptime)
+                ).replace(microsecond=0)
+            else:
+                new_uptime = (datetime.utcnow() - timedelta(seconds=uptime)).replace(
+                    microsecond=0
+                )
+                if abs((new_uptime - self._last_uptime).total_seconds()) > 5:
+                    data[ATTR_UPTIME] = self._last_uptime = new_uptime
+                else:
+                    data[ATTR_UPTIME] = self._last_uptime
         if self._legacy:
             if self._kind == "laser":
                 data.update(
@@ -177,6 +194,11 @@ class Brother:  # pylint:disable=too-many-instance-attributes
         """Return True is data is available."""
         return bool(self.data)
 
+    def shutdown(self):
+        """Unconfigure SNMP engine."""
+        if self._snmp_engine:
+            lcd.unconfigure(self._snmp_engine, None)
+
     async def _get_data(self):
         """Retreive data from printer."""
         raw_data = {}
@@ -196,12 +218,9 @@ class Brother:  # pylint:disable=too-many-instance-attributes
             errindication, errstatus, errindex, restable = await hlapi.getCmd(
                 *request_args, *self._oids
             )
-        except PySnmpError as error:
+        except PySnmpError as err:
             self.data = {}
-            raise ConnectionError(error)
-        finally:
-            # unconfigure SNMP engine
-            lcd.unconfigure(self._snmp_engine, None)
+            raise ConnectionError(err) from err
         if errindication:
             self.data = {}
             raise SnmpError(errindication)
@@ -288,7 +307,7 @@ class SnmpError(Exception):
 
     def __init__(self, status):
         """Initialize."""
-        super(SnmpError, self).__init__(status)
+        super().__init__(status)
         self.status = status
 
 
@@ -297,5 +316,5 @@ class UnsupportedModel(Exception):
 
     def __init__(self, status):
         """Initialize."""
-        super(UnsupportedModel, self).__init__(status)
+        super().__init__(status)
         self.status = status
